# -*- coding: utf-8 -*-
"""
wcm.

:license: Apache 2.0
"""

import configparser
import logging
import os
import sys
from pathlib import Path

import click

import semver

import wcm
from wcm import _component, _utils, _download, _list

__DEFAULT_WCM_CREDENTIALS_FILE__ = "~/.wcm/credentials"


@click.group()
@click.option("--verbose", "-v", default=0, count=True)
def cli(verbose):
    _utils.init_logger()
    lv = ".".join(_utils.get_latest_version().split(".")[:3])
    cv = ".".join(wcm.__version__.split(".")[:3])

    if semver.compare(lv, cv) > 0:
        click.secho(
            f"""WARNING: You are using wcm version {wcm.__version__}, however version {lv} is available.
You should consider upgrading via the 'pip install --upgrade wcm' command.""",
            fg="yellow",
        )


@cli.command(help="Show wcm version.")
def version(debug=False):
    click.echo(f"{Path(sys.argv[0]).name} v{wcm.__version__}")


@cli.command(help="Initialize a directory for a new component.")
@click.option("--yes", "-y", is_flag=True)
@click.argument(
    "component",
    type=click.Path(file_okay=False, dir_okay=True, writable=True, exists=True),
    default=".",
)
def init(component, yes=False):
    component_name = Path(".").resolve().name
    version = "0.0.0"
    description = ""
    author = ""
    license = "Apache"

    if not yes:
        component_name = click.prompt("Component Name", default=component_name)
        version = click.prompt("Version", default=version)
        description = click.prompt("Description", default=description)
        author = click.prompt("Author", default=author)
        license = click.prompt("License", default=license)
    spec = f"""name: {component_name}
version: {version}
description: {description}
author: {author}
license: {license}
"""
    click.echo(spec)
    if yes or click.confirm("Do you want to continue?"):
        try:
            with Path("wings-component.yml").open("w") as fh:
                fh.write(spec)
                click.secho(f"Success", fg="green")
        except FileNotFoundError:
            with Path("wings-component.yaml").open("w") as fh:
                fh.write(spec)
                click.secho(f"Success", fg="green")
    else:
        click.secho(f"Aborted!", fg="red")


@cli.command(help="Configure credentials")
@click.option(
    "--profile",
    "-p",
    envvar="WCM_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
def configure(profile="default"):
    server_wings = click.prompt("WINGS Server URL")
    export_wings_url = click.prompt("WINGS Export URL")
    user_wings = click.prompt("WINGS User")
    password_wings = click.prompt("WINGS Password", hide_input=True)
    domain_wings = click.prompt("WINGS Domain")

    credentials_file = Path(
        os.getenv("WCM_CREDENTIALS_FILE", __DEFAULT_WCM_CREDENTIALS_FILE__)
    ).expanduser()
    os.makedirs(str(credentials_file.parent), exist_ok=True)

    credentials = configparser.ConfigParser()
    credentials.optionxform = str

    if credentials_file.exists():
        credentials.read(credentials_file)

    credentials[profile] = {
        "serverWings": server_wings,
        "exportWingsURL": export_wings_url,
        "userWings": user_wings,
        "passwordWings": password_wings,
        "domainWings": domain_wings,
    }

    with credentials_file.open("w") as fh:
        credentials_file.parent.chmod(0o700)
        credentials_file.chmod(0o600)
        credentials.write(fh)
        click.secho(f"Success", fg="green")


@cli.command(help="Deploy the pacakge to the wcm.")
@click.option("--debug/--no-debug", "-d/-nd", default=False)
@click.option("--dry-run", "-n", is_flag=True)
@click.option("--ignore-data/--no-ignore-data", "-i/-ni", default=False)
@click.option("--force", "-f", is_flag=True, help="Replace existing components")
@click.option(
    "--profile",
    "-p",
    envvar="WCM_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
@click.argument(
    "component",
    type=click.Path(file_okay=False, dir_okay=True, writable=True, exists=True),
    default=".",
)
def publish(component, profile="default", debug=False, dry_run=False, ignore_data=False, force=False):
    logging.info("Publishing component")
    _component.deploy_component(
        component, profile=profile, debug=debug, dry_run=dry_run, ignore_data=ignore_data, force=force
    )

    click.secho(f"Success", fg="green")


@cli.command(help="Download a component from wings server. Data stored in .yaml file and source code downloaded to folder within same directory. file-path can be specified to download into a specific directory")
@click.option(
    "--profile",
    "-p",
    envvar="WCM_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
@click.option(
    "--file-path",
    "-f",
    type=str,
    default=None,
)
@click.argument("component_id", default=None, type=str)
def download(component_id, profile="default", file_path=None):
    logging.info("Downloading component")
    _download.download(component_id, profile=profile, download_path=file_path)
    click.secho(f"Download complete", fg="green")


@cli.command(help="Lists all the components in the current wings instance")
@click.option(
    "--profile",
    "-p",
    envvar="WCM_PROFILE",
    type=str,
    default="default",
    metavar="<profile-name>",
)
def list(profile="default"):
    _list.list_components(profile=profile)
    click.secho(f"Done", fg="green")



# -*- coding: utf-8 -*-

import logging

from jsonschema import Draft7Validator

schema = {
    "type": "object",
    "required": ["name", "version"],
    "properties": {
        "name": {
            "description": "The name of the component.",
            "type": "string",
            "maxLength": 214,
            "minLength": 1,
            "pattern": "^(?:@[a-zA-Z0-9-~][a-zA-Z0-9-._~]*/)?[a-zA-Z0-9-~][a-zA-Z0-9-._~]*$",
        },
        "version": {"type": "string"},
        "description": {"type": "string"},
        "keywords": {"type": "array", "items": {"type": "string"}},
        "homepage": {"type": "string", "oneOf": [{"format": "uri"}, {"enum": ["."]}]},
        "bugs": {
            "type": ["object", "string"],
            "properties": {
                "url": {
                    "type": "string",
                    "description": "The url to your project's issue tracker.",
                    "format": "uri",
                },
                "email": {"type": "string", "format": "email"},
            },
        },
        "license": {"type": "string"},
        "dateCreated": {"type": "string"},
        "author": {"type": "array", "items": {"$ref": "#/definitions/person"}},
        "contributors": {"type": "array", "items": {"$ref": "#/definitions/person"}},
        "maintainers": {"type": "array", "items": {"$ref": "#/definitions/person"}},
        "publisher": {"$ref": "#/definitions/organization"},
        "assumptions": {"type": "string"},
        "citation": {"type": "string"},
        "memoryRequirements": {"type": "string"},
        "operatingSystems": {"type": "array", "items": {"type": "string"}},
        "processorRequirements": {"type": "string"},
        "softwareRequirements": {"type": "string"},
        "repository": {
            "type": ["object", "string"],
            "properties": {
                "type": {"type": "string"},
                "url": {"type": "string"},
                "directory": {"type": "string"},
            },
        },
        "container": {
            "type": ["object", "string"],
            "properties": {
                "dockerPull": {"type": "string"},
                "directory": {"type": "string"},
            },
        },
        "source": {"type": "string"},
        "schemaVersion": {"type": "string"},
        "wings": {
            "type": ["object"],
            "properties": {
                "inputs": {"type": "array", "items": {"$ref": "#/definitions/ioData"}},
                "outputs": {"type": "array", "items": {"$ref": "#/definitions/ioData"}},
                "rules": {"type": ["string","array"]},
                "inheritedRules": {"type": ["string", "array"]},
                "documentation": {"type": "string"},
                "requirement": {"type": "object", "items": {"$ref": "/definitions/requirements"}},
                "componentType": {"type": "string"},
                "files": {"type": "array", "items": {"type": "string"}},
            },
        },
    },

    "definitions": {
        "person": {
            "type": ["object", "string"],
            "required": ["name"],
            "properties": {
                "name": {"type": "string"},
                "url": {"type": "string", "format": "uri"},
                "email": {"type": "string", "format": "email"},
            },
        },
        "ioData": {
            "type": ["object"],
            "required": ["role", "prefix", "isParam", "type", "dimensionality"],
            "properties": {
                "role": {"type": "string"},
                "prefix": {"type": "string"},
                "isParam": {"type": "boolean"},
                "type": {"type": "string"},
                "dimensionality": {"type": "integer"}
            },
        },
        "requirement": {
            "type": ["object"],
            "properties": {
                "storageGB": {"type": "double"},
                "memoryGB": {"type": "double"},
                "need64bit": {"type": "boolean"},
                "softwareIds": {"type": "array", "items": {"type": "string"}},

            },
        },
        "organization": {
            "type": ["object", "string"],
            "required": ["name"],
            "properties": {
                "name": {"type": "string"},
                "url": {"type": "string", "format": "uri"},
            },
        },
    },

}
#Missing extension for variables of each input and variable of each output

v = Draft7Validator(schema)


def _msg(e):
    """Generate a user friendly error message."""
    return e.message


def check_package_spec(spec):
    """Check package specification."""
    err = []
    for e in v.iter_errors(spec):
        err.append(_msg(e))
        logging.error(_msg(e))

    if err:
        raise ValueError("Invalid component specification.")



import argparse
import concurrent.futures
import configparser
from contextlib import contextmanager
import yaml
import yamlordereddictloader
import logging
import json
import wings
import click
import os
import zipfile
import shutil
from wcm import _schema, _utils

logger = logging.getLogger()


@contextmanager
def _cli(**kw):
    i = None
    try:
        logger.debug("Initializing WINGS API Client")
        i = wings.init(**kw)
        yield i
    finally:
        if i:
            i.close()


def download(component_dir, profile=None, download_path=None):

    comp_id = component_dir

    # sets path, this determines where the component will be downloaded. Default is the current directory of the program
    if download_path is None:
        path = os.getcwd()
    else:
        path = download_path

    with _cli(profile=profile) as wings_instance:
        component = wings_instance.component.get_component_description(comp_id)

        if component is None:
            logger.error("Invalid ID: \"" + comp_id + "\"")
            exit(1)

        # Make new folder to put everything in
        path = os.path.join(path, comp_id)

        if os.path.exists(path):
            click.echo("\"" + path + "\" already exists. Do you want to overwrite it? [y/n]")
            ans = input()
            if ans == 'y' or ans == "yes":
                shutil.rmtree(path)
            else:
                logger.info("Aborting Download")
                exit(0)

        os.mkdir(path)

        wings_instance.component.download_component(comp_id, os.path.join(path, "components"))

        yaml_data = {}
        data_types = {}

        yaml_data["name"] = ""
        yaml_data["version"] = ""
        # yaml_data["#description"] = None
        # yaml_data["#keywords"] = None
        # yaml_data["homepage"] = None
        # amlData["license"] = None
        # yaml_data["author"] = None
        # yaml_data["container"] = None
        # yaml_data["repository"] = None

        yaml_data["wings"] = component
        component = yaml_data["wings"]

        # takes the id and splits it by the '#' sign
        # (id example: http://localhost:8080/export/users/mint/api-test/components/library.owl#HAND-1)
        info = component["id"].split("#")
        info = info[len(info) - 1]  # gets the last index of the split (ie: HAND-1)
        info = info.split("-")  # splits it by the '-' (ie {"HAND","1"})

        # First part becomes name, other becomes version
        yaml_data["name"] = info[0]
        if len(info) > 1:
            yaml_data["version"] = info[-1]
        else:
            logger.warning("No version could be ascertained from the name")

        try:
            component.pop("location")
            component.pop("id")
            component.pop("type")
            component["documentation"] = component["documentation"].strip()
            component["files"] = ["src\\*"]
        except KeyError:
            logger.warning("Component seems to be missing metadata")

        # loops through every input field
        if len(component["inputs"]) <= 0:
            logger.warning("Component has no inputs")
        for i in (component["inputs"]):
            files = {}
            i.pop("id")
            try:
                if "XMLSchema" not in (i["type"]):
                    type_name = i["type"].split("#")
                    type_name = type_name[len(type_name) - 1]

                    i["type"] = "dcdom:" + type_name

                    files["files"] = ["data/placeholder.tif"]
                    data_types[type_name] = files
            except:
                logger.warning("no type in " + str(i))

        component["data"] = data_types

        if len(component["outputs"]) <= 0:
            logger.warning("Component has no outputs")
        for o in (component["outputs"]):
            files = {}
            o.pop("id")
            try:
                if "XMLSchema" not in o["type"]:
                    type_name = o["type"].split("#")
                    type_name = type_name[len(type_name) - 1]

                    o["type"] = "dcdom:" + type_name

                    files["files"] = ["data/placeholder.tif"]
                    data_types[type_name] = files
            except:
                logger.warning("no type in " + str(o))

        # makes the YAML file
        stream = open(os.path.join(path, "wings-component.yaml"), 'w+')
        yaml.dump(yaml_data, stream, sort_keys=False)

        logger.info("Generated YAML")

        # makes the src folder in the directory
        try:
            os.mkdir(os.path.join(path, "src"))
        except FileExistsError:
            logger.warning("src folder already exists")

        data_path = os.path.join(path, "data")

        try:
            os.mkdir(data_path)
        except:
            logger.warning("data folder already exists")

        open(os.path.join(data_path, "placeholder.tif"), 'w+')

        # unzip components
        comp_os_path = ""
        try:
            comp_os_path = os.path.join(path, "components")
            zip_path = os.path.join(comp_os_path, comp_id + ".zip")
            with zipfile.ZipFile(zip_path, "r") as zip_ref:
                zip_ref.extractall(comp_os_path)
        except zipfile.BadZipFile:
            logger.error("Downloaded zip file seems to be corrupt")
            exit(1)

        # copy files into src folder
        comp_files = os.listdir(os.path.join(comp_os_path, comp_id))
        for files in comp_files:
            full_file_name = os.path.join(os.path.join(comp_os_path, comp_id), files)
            if os.path.isfile(full_file_name):
                shutil.copy(full_file_name, os.path.join(path, "src"))

        # remove component folder
        shutil.rmtree(comp_os_path)

        logger.info("Download complete")


def _main():
    parser = argparse.ArgumentParser(
        description="Download wings components given the component id."
    )
    parser.add_argument(
        "--file-path",
        "-f",
        type=str,
        default=None,
    )
    parser.add_argument("-c", "--component", required=True, help="Component name to download")
    parser.add_argument(
        "-d", "--debug", dest="debug", default=False, action="store_true", help="Debug"
    )
    parser.add_argument(
        "--dry-run", dest="dry_run", default=False, action="store_true", help="Dry run"
    )
    args = parser.parse_args()

    if args.debug:
        os.environ["WINGS_DEBUG"] = "1"
    _utils.init_logger()
    component_dir = args.component
    file_path = args.file_path
    download(component_dir, file_path)


if __name__ == "__main__":
    try:
        _main()
    except Exception as e:
        logger.exception(e)



#!/usr/bin/env python3
"""Component Uploader."""

import argparse
import logging
import os
from contextlib import contextmanager
from pathlib import Path
from shutil import make_archive

import wings
from semver import parse_version_info
from yaml import load
import click

from wcm import _schema, _utils

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

log = logging.getLogger()


@contextmanager
def _cli(**kw):
    i = None
    try:
        log.debug("Initializing WINGS API Client")
        i = wings.init(**kw)
        yield i
    finally:
        if i:
            i.close()


def check_data_types(spec):
    _types = set()
    for _t in spec["inputs"]:
        if not _t["isParam"] and _t["type"] not in _types:
            dtype = _t["type"][6:]
            if dtype not in spec.get("data", {}):
                raise ValueError(f"data-type {dtype} not defined")

    for _t in spec["outputs"]:
        if not _t["isParam"] and _t["type"] not in _types:
            if dtype not in spec.get("data", {}):
                raise ValueError(f"data-type {dtype} not defined")


def create_data_types(spec, component_dir, cli, ignore_data):
    for dtype, _file in spec.get("data", {}).items():
        cli.data.new_data_type(dtype, None)
        if ignore_data:
            continue

        if _file:
            # Properties
            format = _file.get("format", None)
            metadata_properties = _file.get("metadataProperties", {})
            if metadata_properties or format:
                cli.data.add_type_properties(
                    dtype, properties=metadata_properties, format=format
                )

            # Files
            for f in _file.get("files", ()):
                cli.data.upload_data_for_type(
                    (component_dir / Path(f)).resolve(), dtype
                )


def check_if_component_exists(spec, profile, force, creds):
    with _cli(profile=profile, **creds) as wi:
        if spec["version"].isspace() or len(spec["version"]) <= 0:
            name = spec["name"]
        else:
            name = spec["name"] + "-" + spec["version"]

        comps = wi.component.get_component_description(name)
        if not comps is None and not force:
            click.echo("publishing this will override an existing component. Continue anyway? [y/n]")
            ans = input()
            if ans == "n" or ans == "no":
                log.info("Aborting publish")
                exit(0)


def deploy_component(component_dir, profile=None, creds={}, debug=False, dry_run=False, ignore_data=False, force=False):
    component_dir = Path(component_dir)
    if not component_dir.exists():
        raise ValueError("Component directory does not exist.")

    with _cli(profile=profile, **creds) as cli:
        try:
            spec = load((component_dir / "wings-component.yml").open(), Loader=Loader)
        except FileNotFoundError:
            spec = load((component_dir / "wings-component.yaml").open(), Loader=Loader)

        try:
            _schema.check_package_spec(spec)
        except ValueError as err:
            log.error(err)
            exit(1)

        check_if_component_exists(spec, profile, force, creds)

        name = spec["name"]
        version = spec["version"]

        # _id = f"{name}-v{version}" #removed this line because it would make errors if 'v' was in version name
        if version.isspace() or len(version) <= 0:
            log.warning("No version. Component will be uploaded with no version identifier")
            _id = name
        else:
            _id = name + "-" + version

        if ignore_data:
            log.info("Upload data and metadata skipped")

        wings_component = spec["wings"]
        log.debug("Check component's data-types")
        check_data_types(wings_component)
        log.debug("Create component's data-types")
        create_data_types(wings_component, component_dir, cli, ignore_data)

        log.debug("Create component's type")
        cli.component.new_component_type(wings_component["componentType"], None)

        log.debug("Create the component")
        cli.component.new_component(_id, wings_component["componentType"])

        log.debug("Create component's I/O, Documentation, etc.")
        cli.component.save_component(_id, wings_component)

        try:
            _c = make_archive("_c", "zip", component_dir / "src")
            log.debug("Upload component code")
            cli.component.upload_component(_c, _id)
            return cli.component.get_component_description(_id)
        finally:
            os.remove(_c)



def _main():
    parser = argparse.ArgumentParser(
        description="Run WINGS template based on simulation matrix."
    )
    parser.add_argument(
        "-w",
        "--wings-config",
        dest="wings_config",
        required=True,
        help="WINGS Configuration File",
    )
    parser.add_argument(
        "-d", "--debug", dest="debug", default=False, action="store_true", help="Debug"
    )
    parser.add_argument(
        "--dry-run", dest="dry_run", default=False, action="store_true", help="Dry run"
    )
    parser.add_argument("component_dir", help="Component Directory")
    args = parser.parse_args()

    if args.debug:
        os.environ["WINGS_DEBUG"] = "1"
    _utils.init_logger()

    deploy_component(**vars(args))


if __name__ == "__main__":
    try:
        _main()
        log.info("Done")
    except Exception as e:
        log.exception(e)



# -*- coding: utf-8 -*-

import logging
import os
import requests


def init_logger():
    logger = logging.getLogger()
    handler = logging.StreamHandler()
    formatter = logging.Formatter("%(asctime)s %(name)-12s %(levelname)-8s %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG if os.getenv("WINGS_DEBUG", False) else logging.INFO)


def get_latest_version():
    return requests.get("https://pypi.org/pypi/wcm/json").json()["info"]["version"]



# -*- coding: utf-8 -*-

import os

from setuptools import find_packages, setup

install_requires = [
    "Click>=7.0",
    "PyYAML",
    "yamlordereddictloader",
    "jsonschema>=3.0.0",
    "semver>=2.8.1",
    "wings",
    "requests",
]


# Utility function to read the README file.
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def find_package_data(dirname):
    def find_paths(dirname):
        items = []
        for fname in os.listdir(dirname):
            path = os.path.join(dirname, fname)
            if os.path.isdir(path):
                items += find_paths(path)
            elif not path.endswith(".py") and not path.endswith(".pyc"):
                items.append(path)
        return items

    items = find_paths(dirname)
    return [os.path.relpath(path, dirname) for path in items]


version = {}
with open("src/wcm/__init__.py") as fp:
    exec(fp.read(), version)


setup(
    name="wcm",
    version=version["__version__"],
    author="Rajiv Mayani",
    author_email="mayani@isi.edu",
    description=__doc__,
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    license="Apache-2",
    url="https://github.com/mintproject/wcm",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
    ],
    entry_points={"console_scripts": ["wcm = wcm.__main__:cli"]},
    package_dir={"": "src"},
    packages=find_packages(where="src", exclude=["wcm.tests*"]),
    package_data={"wcm": find_package_data("src/wcm")},
    exclude_package_data={"wcm": ["tests/*"]},
    zip_safe=False,
    install_requires=install_requires,
    python_requires=">=3.5.0",
)



import logging
import json
import wings
import os
import click
from wcm import _schema, _utils
from contextlib import contextmanager

logger = logging.getLogger()


@contextmanager
def _cli(**kw):
    i = None
    try:
        logger.debug("Initializing WINGS API Client")
        i = wings.init(**kw)
        yield i
    finally:
        if i:
            i.close()


def list_components(profile="default"):
    outp = ""
    with _cli(profile=profile) as wings_instance:
        component = wings_instance.component.get_all_items()
        component = component["children"]
        for i in component:
            try:
                comp_class = ((i["cls"])["component"])["id"]
                comp_class = comp_class.split('#')

                outp += "[" + comp_class[-1] + "]\n"

                length = len(i["children"])
                count = 1

                if length > 0:
                    outp += "  └─┐\n"

                for j in i["children"]:
                    comp_id = ((j["cls"])["component"])["id"]
                    comp_id = comp_id.split('#')

                    if length == 1 or count == length:
                        outp += "    └─ " + comp_id[-1] + "\n"
                    else:
                        outp += "    ├─ " + comp_id[-1] + "\n"
                    count += 1

                outp += "\n"
            except:
                 logger.error("Wings error: Maybe, the component is corrupted.")

        click.echo(outp)


def _main():
    list_components()


if __name__ == "__main__":
    try:
        _main()
    except Exception as e:
        logger.exception(e)
